/*
 * ws2812.S
 *
 * Created: 17.12.2017
 *  Author: Falk Brunner
 */
 
.nolist
#include <avr/io.h>	
.list

// select IR-Data PIN and bit

IR_PIN = PINL
IR_BIT = PL1

// register name assignment

fifo_base  =  8         ; + r9
fifo_top   = 10         ; + r11
fifo_struct= 12         ; + r13
isr_cnt    = 16     
bit_cnt    = 17 
mask       = 18
led_data   = 19
port_set   = 20
port_clr   = 21
data_cnt   = 22         ; + r23
tmp        = 24
ir_data    = 25
port_ptr   = 26         ; X
fifo_write = 28         ; Y
led_ptr    = 30         ; Z

.extern cnt_2kHz

/*
C declaration 
void ws2812_write(uint8_t *p_data, uint16_t cnt, uint8_t *port, uint8_t mask);
registers                 r24/r25       r22/r23        r20/r21       r18/r19
*/

.global ws2812_write
ws2812_write:
    push    r17
    push    r26
    push    r27
    clr     r1

    cp      data_cnt, r1             
    cpc     data_cnt+1, r1
    brne    ws2812_w_go  
    rjmp    ws2812_w_end    ; skip function, when cnt==00
ws2812_w_go:
    movw    led_ptr, r24    ; data pointer to Z, free r24/r25
    movw    port_ptr, r20   ; port pointer to X, free r20/r21

    ld      port_set, X
    or      port_set, mask  ; port data with bit set
    com     mask
    mov     port_clr, port_set
    and     port_clr, mask  ; port data with bit clear
ws2812_w_byte_loop:
    ldi     bit_cnt, 8
    ld      led_data, Z+     ; load next data
    cli                     
ws2812_w_bit_loop:
    st      X, port_set
    lsl     led_data
    brcc    ws2812_w_bit_is_0   ; test MSB    
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    st      X, port_clr
    rjmp    ws2812_w_bit_end
ws2812_w_bit_is_0:
    nop
    st      X, port_clr
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
ws2812_w_bit_end:
    subi    bit_cnt, 1
    brne    ws2812_w_bit_loop

    ; use this additiona delay loop to test for maximum tolerated time between bits/bytes
/*    
; test delay
    ldi     mask, 30
delay:
    subi    mask, 1
    brne    delay
*/  
    ; enable here interupts only, when LEDs can tolarate a longer break in the data signal!
    ; WS2812B can't!!!
    ;sei                 
    subi    data_cnt, 1
    sbc     data_cnt+1, r1     
    brne    ws2812_w_byte_loop

ws2812_w_end:        
    pop     r27
    pop     r26
    pop     r17
	ret

/*
C declaration 
void ws2812_write_and_sample(uint8_t *p_data, uint16_t cnt, uint8_t *port, uint8_t mask, fifo_t *fifo);
registers                            r24/r25       r22/r23        r20/r21       r18/r19       r16/r17
*/

.global ws2812_write_and_sample
ws2812_write_and_sample:
    push    r8
    push    r9
    push    r10
    push    r11
    push    r12
    push    r13
    push    r16
    push    r17
    push    r26
    push    r27
    push    r28
    push    r29

    clr     r1
    cp      data_cnt, r1             
    cpc     data_cnt+1, r1
    brne    ws2812_ws_go  
    rjmp    ws2812_ws_end       // exit function, when cnt==00
ws2812_ws_go:
    movw    led_ptr, r24        ; data pointer to Z, free r24/r25
    movw    fifo_write, r16     ; fifo struct pointer to Y, free r20/r21
    movw    fifo_struct, r16    ; save fifo struct pointer for later
    movw    port_ptr, r20       ; port pointer to X, free r16/r17
    ldd     fifo_base,Y+4       ; fifo base pointer
    ldd     fifo_base+1,Y+5
    ldd     fifo_top,Y+6        ; fifo top pointer
    ldd     fifo_top+1,Y+7
    ldd     tmp,Y+0             ; fifo write pointer
    ldd     tmp+1,Y+1
    movw    fifo_write, tmp     ; fifo write pointer into Y
    ld      port_set, X         ; prepare port data for fast IO set/clear
    or      port_set, mask      ; port data with bit set
    com     mask
    mov     port_clr, port_set
    and     port_clr, mask      ; port data with bit clear
    ldi     mask, 0x80          ; mask is now used for sampling data storage
    ldi     ir_data, 0
    ldi     isr_cnt, 0
ws2812_ws_byte_loop:
    ldi     bit_cnt, 8
    ld      led_data, Z+        ; load next data
ws2812_ws_bit_loop:
    st      X, port_set
    lsl     led_data
    brcc    ws2812_ws_bit_is_0  ; test MSB    
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    st      X, port_clr
    rjmp    ws2812_ws_bit_end
ws2812_ws_bit_is_0:
    nop
    st      X, port_clr
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
ws2812_ws_bit_end:
    subi    bit_cnt, 1
    brne    ws2812_ws_bit_loop

    ; test timer flag
    sbis    _SFR_IO_ADDR(TIFR1), OCF1A
    rjmp    ws2812_ws_sample_end
    ldi     tmp, (1<<OCF1A)
    out     _SFR_IO_ADDR(TIFR1), tmp    ; clear OCF1A
    ; sample IR data
    lds     tmp, IR_PIN
    sbrc    tmp, IR_BIT
    or      ir_data, mask
    lsr     mask
    brne    ws2812_ws_sample_end
    ; 8 bits sampled, store in FIFO
    st      Y+, ir_data
    ldi     ir_data, 0
    inc     isr_cnt
    ldi     mask, 0x80
    cp      fifo_top, fifo_write        ; fifo_top >= fifo_write ?
    cpc     fifo_top+1, fifo_write+1
    brsh    ws2812_ws_sample_end
    movw    fifo_write, fifo_base       ; no, pointer wrap around   
ws2812_ws_sample_end:     
    subi    data_cnt, 1
    sbc     data_cnt+1, r1     
    brne    ws2812_ws_byte_loop

    ; continue sampling until an full byte
    cpi     mask, 0x80
    breq    ws2812_ws_sample_finish_end

ws2812_ws_sample_finish:
    ; test timer flag
    sbis    _SFR_IO_ADDR(TIFR1), OCF1A
    rjmp    ws2812_ws_sample_finish
    ldi     tmp, (1<<OCF1A)
    out     _SFR_IO_ADDR(TIFR1), tmp    ; clear OCF1A
    ; sample IR data
    lds     tmp, IR_PIN
    sbrc    tmp, IR_BIT
    or      ir_data, mask
    lsr     mask
    brne    ws2812_ws_sample_finish
    ; 8 bits sampled, store in FIFO
    st      Y+, ir_data
    inc     isr_cnt
    cp      fifo_top, fifo_write        ; fifo_top >= fifo_write ?
    cpc     fifo_top+1, fifo_write+1
    brsh    ws2812_ws_sample_finish_end
    movw    fifo_write, fifo_base       ; no, pointer wrap around   
ws2812_ws_sample_finish_end: 
    movw    r30, fifo_struct
    std     Z+0, fifo_write
    std     Z+1, fifo_write+1           ; write back fifo write pointer
    lds     tmp, cnt_2kHz               
    add     tmp, isr_cnt
    sts     cnt_2kHz, tmp               ; correct ISR counter

ws2812_ws_end:        
    pop     r29
    pop     r28
    pop     r27
    pop     r26
    pop     r17
    pop     r16
    pop     r13
    pop     r12
    pop     r11
    pop     r10
    pop     r9
    pop     r8
	ret

